@@ -2,11 +2,11 @@
 use chrono::{DateTime, Duration, Local};
 use clap::Parser;
 use colored::Colorize;
+use reqwest::blocking::Client;
 use scraper::{Html, Selector};
-use std::error;
 use std::fs::OpenOptions;
 use std::io::Write;
-use url::Url;
+use url::{ParseError, Url};
 
 #[derive(Parser, Debug)]
 #[clap(about, version, author)]
@@ -16,6 +16,25 @@ struct Cli {
     #[clap(short, long, value_name = "URL")]
     /// URL(s) to ignore if found (full url); separate multiple URLs with a comma
     skip: Option<String>,
+    #[clap(short, long)]
+    /// Quick run - check links only on the URL you provide and not any subsequent links
+    quick: bool,
+}
+
+struct GoodLink {
+    url: Url,
+    html: String,
+}
+
+impl GoodLink {
+    fn new(url: Url, html: String) -> GoodLink {
+        GoodLink { url, html }
+    }
+}
+
+enum CheckLinkResponse {
+    Good(GoodLink),
+    Bad,
 }
 
 #[derive(PartialEq, Clone)]
@@ -57,16 +76,9 @@ fn get_abs_link(href: &str, referer: Url) -> String {
     }
 }
 
-/// get all links from one url
-fn get_links(url: &Url) -> Result<Vec<Link>, Box<dyn error::Error>> {
-    let res = reqwest::blocking::get(url.as_ref())?;
-
-    // current url, even if redirected
-    let referer: &Url = &res.url().clone();
-
-    // get and parse the html
-    let html = res.text()?;
-    let document = Html::parse_document(&html);
+/// get all links from one url's html response
+fn get_links(referer: GoodLink) -> Result<Vec<Link>, ParseError> {
+    let document = Html::parse_document(&referer.html);
     let selector = Selector::parse("a").unwrap();
 
     let mut links = vec![];
@@ -76,12 +88,12 @@ fn get_links(url: &Url) -> Result<Vec<Link>, Box<dyn error::Error>> {
         // if it has an href value, modify that value as necessary and push to links
         if let Some(mut v) = element.value().attr("href") {
             v = v.trim();
-            // don't add mailto links or anchor links
+            // don't add mailto links or relative anchor links
             if v.starts_with("mailto") || v.starts_with('#') {
                 continue;
             } else {
-                let link = Url::parse(&get_abs_link(v, referer.clone()))?;
-                links.push(Link::new(referer.clone(), link));
+                let link = Url::parse(&get_abs_link(v, referer.url.clone()))?;
+                links.push(Link::new(referer.url.clone(), link));
             }
         }
     }
@@ -90,9 +102,15 @@ fn get_links(url: &Url) -> Result<Vec<Link>, Box<dyn error::Error>> {
 }
 
 /// check validity of individual link
-fn check_link(url: &Url) -> Result<bool, reqwest::Error> {
-    let res = reqwest::blocking::get(url.as_ref())?;
-    Ok(res.status() != reqwest::StatusCode::NOT_FOUND)
+fn check_link(url: &Url, client: &Client) -> Result<CheckLinkResponse, reqwest::Error> {
+    let res = client.get(url.as_ref()).send()?;
+    match res.status() != reqwest::StatusCode::NOT_FOUND {
+        true => {
+            let good_link = GoodLink::new(res.url().clone(), res.text()?);
+            Ok(CheckLinkResponse::Good(good_link))
+        }
+        false => Ok(CheckLinkResponse::Bad),
+    }
 }
 
 fn main() {
@@ -127,6 +145,10 @@ fn main() {
     let errors_csv = format!("{}_errors_{}.csv", domain, dt.format(format).to_string());
 
     println!("\nChecking links found in {:?}", url.to_string());
+    if args.quick {
+        println!("{}", "Note: this is a quick run.".yellow());
+    }
+    println!();
 
     // start the check
     let start: DateTime<Local> = Local::now();
@@ -136,10 +158,13 @@ fn main() {
     let mut errors: Vec<CheckError> = vec![];
 
     let mut count = 0;
+    let mut total_count = 0;
 
-    while !unchecked_links.is_empty() {
+    let client = Client::new();
+
+    while let Some(content) = unchecked_links.pop() {
         count += 1;
-        let content = unchecked_links.pop().unwrap();
+        total_count += 1;
 
         // if we already know the link is valid, we've already fetched all links from it, so no
         // need to process it (but we can't skip known bad links, because we need their referer)
@@ -153,11 +178,17 @@ fn main() {
         }
 
         // otherwise check it
-        match check_link(&content.link) {
+        match check_link(&content.link, &client) {
             // the link is ok
-            Ok(true) => {
+            Ok(CheckLinkResponse::Good(good_link)) => {
                 println!("{} is {}", content.link, "OK".green());
                 valid_links.push(content.clone());
+
+                // only get links from the starting url if this is a "quick" run
+                if args.quick && count > 1 {
+                    continue;
+                }
+
                 // get further links if link is within domain we're interested in & not a file
                 // TODO: probably a better way to check if a file than this
                 if content.referer.host() == content.link.host()
@@ -172,7 +203,7 @@ fn main() {
                     && !&content.link.as_ref().ends_with(".jpg")
                     && !&content.link.as_ref().ends_with(".jpeg")
                 {
-                    match get_links(&content.link) {
+                    match get_links(good_link) {
                         Ok(new_urls) => {
                             'outer: for new_url in new_urls {
                                 // skip any urls specified by user
@@ -191,15 +222,19 @@ fn main() {
                                     // if we already know the link is broken, add to invalid
                                     if invalid_links.iter().any(|link| link.link == new_url.link) {
                                         invalid_links.push(new_url.clone());
+                                        total_count += 1;
                                     // if we already know the link is good, add to valid
                                     } else if valid_links
                                         .iter()
                                         .any(|link| link.link == new_url.link)
                                     {
                                         valid_links.push(new_url.clone());
+                                        total_count += 1;
                                     } else {
                                         unchecked_links.push(new_url.clone());
                                     }
+                                } else {
+                                    total_count += 1;
                                 }
                             }
                         }
@@ -221,7 +256,7 @@ fn main() {
                 }
             }
             // the link is bad
-            Ok(false) => {
+            Ok(CheckLinkResponse::Bad) => {
                 invalid_links.push(content.clone());
                 println!("{} is {}", content.link, "Broken".red());
             }
@@ -262,7 +297,13 @@ fn main() {
         human_elapsed = format!("{} seconds", elapsed.num_seconds());
     }
 
-    println!("\nChecked {} links in {}.", count, human_elapsed,);
+    println!(
+        "\nChecked {} links (verifying an approximate total of {}) in {}.",
+        count,
+        // (valid_links.len() + invalid_links.len() + errors.len()),
+        total_count,
+        human_elapsed,
+    );
 
     // process results
     if invalid_links.is_empty() {
